﻿Public Class CarComparer
    Implements IComparer(Of Car)

    ' Different kinds of sorts.
    Public Enum SortType
        ByName
        ByMaxSpeed
        ByHorsepower
        ByPrice
    End Enum

    ' The current sort style.
    ' The property is read-only so must be set via the constructor.
    Private _Sort As SortType
    Private ReadOnly Property Sort As SortType
        Get
            Return _Sort
        End Get
    End Property

    ' Initializing constructor.
    Public Sub New(ByVal newSort As SortType)
        _Sort = newSort
    End Sub

    ' Compare two cars.
    Public Function Compare(ByVal x As Car, ByVal y As Car) As Integer Implements System.Collections.Generic.IComparer(Of Car).Compare
        ' Compare differently depending on the sort type.
        Select (Sort)
            Case SortType.ByName
                Return x.Name.CompareTo(y.Name)
            Case SortType.ByMaxSpeed
                Return -x.MaximumSpeed.CompareTo(y.MaximumSpeed)
            Case SortType.ByHorsepower
                Return -x.Horsepower.CompareTo(y.Horsepower)
            Case SortType.ByPrice
                Return -x.Price.CompareTo(y.Price)
        End Select

        ' This should never happen (unless we add a
        ' new SortStyle value and forget to update the cases.)
        Throw New Exception("Unknown SortType value")
    End Function
End Class
